#!/usr/bin/env python
# --!-- coding: utf8 --!--
import re

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QTextCursor, QFont, QFontMetrics
from PyQt5.QtWidgets import QAction, qApp, QToolTip, QTextEdit

from manuskript.ui.editors.completer import completer
from manuskript.ui.views.MDEditView import MDEditView
from manuskript.models import references as Ref


class MDEditCompleter(MDEditView):
    def __init__(self, parent=None, index=None, html=None, spellcheck=True, highlighting=False, dict="",
                 autoResize=False):
        MDEditView.__init__(self, parent=parent, index=index, html=html, spellcheck=spellcheck, highlighting=True,
                              dict=dict, autoResize=autoResize)

        self.completer = None
        self.setMouseTracking(True)
        self.refRects = []
        self._noFocusMode = True

        self.textChanged.connect(self.getRefRects)
        self.document().documentLayoutChanged.connect(self.getRefRects)

    def setCurrentModelIndex(self, index):
        MDEditView.setCurrentModelIndex(self, index)
        if self._index and not self.completer:
            self.setCompleter(completer())

    def setCompleter(self, completer):
        self.completer = completer
        self.completer.activated.connect(self.insertCompletion)

    def insertCompletion(self, txt):
        tc = self.textCursor()
        tc.insertText(txt)
        self.setTextCursor(tc)

    def textUnderCursor(self, select=False):
        tc = self.textCursor()
        tc.select(QTextCursor.WordUnderCursor)
        if select:
            self.setTextCursor(tc)
        return tc.selectedText()

    def refUnderCursor(self, cursor):
        pos = cursor.position()
        cursor.select(QTextCursor.BlockUnderCursor)
        text = cursor.selectedText()
        pos -= cursor.selectionStart()
        match = re.findall(Ref.RegExNonCapturing, text)
        for m in match:
            if text.find(m) <= pos <= text.find(m) + len(m):
                return m

                # def event(self, event):
                # if event.type() == QEvent.ToolTip:
                # cursor = self.cursorForPosition(event.pos())
                # ref = self.refUnderCursor(cursor)
                # if ref:
                # QToolTip.showText(self.mapToGlobal(event.pos()), infoForRef(ref))
                # else:
                # QToolTip.hideText()
                # return True
                # return MDEditView.event(self, event)

    def createStandardContextMenu(self):
        menu = MDEditView.createStandardContextMenu(self)

        a = QAction(self.tr("Insert reference"), menu)
        a.triggered.connect(self.popupCompleter)
        menu.insertSeparator(menu.actions()[0])
        menu.insertAction(menu.actions()[0], a)
        return menu

    def keyPressEvent(self, event):
        if self.completer.isVisible():
            if event.key() in (
                    Qt.Key_Enter,
                    Qt.Key_Return,
                    Qt.Key_Escape,
                    Qt.Key_Tab,
                    Qt.Key_Backtab):
                event.ignore()
                return

        isShortcut = (event.modifiers() == Qt.ControlModifier and \
                      event.key() == Qt.Key_Space)

        if not self.completer or not isShortcut:
            self.completer.setVisible(False)
            MDEditView.keyPressEvent(self, event)
            return

        self.popupCompleter()

    def popupCompleter(self):
        if self.completer:
            cr = self.cursorRect()
            cr.moveTopLeft(self.mapToGlobal(cr.bottomLeft()))
            cr.setWidth(self.completer.sizeHint().width())
            self.completer.setGeometry(cr)
            self.completer.popup(self.textUnderCursor(select=True))

    def mouseMoveEvent(self, event):
        MDEditView.mouseMoveEvent(self, event)

        onRef = [r for r in self.refRects if r.contains(event.pos())]

        if not onRef:
            qApp.restoreOverrideCursor()
            QToolTip.hideText()
            return

        cursor = self.cursorForPosition(event.pos())
        ref = self.refUnderCursor(cursor)
        if ref:
            if not qApp.overrideCursor():
                qApp.setOverrideCursor(Qt.PointingHandCursor)
            QToolTip.showText(self.mapToGlobal(event.pos()), Ref.tooltip(ref))

    def mouseReleaseEvent(self, event):
        MDEditView.mouseReleaseEvent(self, event)
        onRef = [r for r in self.refRects if r.contains(event.pos())]
        if onRef:
            cursor = self.cursorForPosition(event.pos())
            ref = self.refUnderCursor(cursor)
            if ref:
                Ref.open(ref)
                qApp.restoreOverrideCursor()

    def resizeEvent(self, event):
        MDEditView.resizeEvent(self, event)
        self.getRefRects()

    def getRefRects(self):
        cursor = self.textCursor()
        f = self.font()
        f.setFixedPitch(True)
        f.setWeight(QFont.DemiBold)
        fm = QFontMetrics(f)
        refs = []
        for txt in re.finditer(Ref.RegEx, self.toPlainText()):
            cursor.setPosition(txt.start())
            r = self.cursorRect(cursor)
            r.setWidth(fm.width(txt.group(0)))
            refs.append(r)
        self.refRects = refs

    def paintEvent(self, event):
        QTextEdit.paintEvent(self, event)

        # Debug: paint rects
        # painter = QPainter(self.viewport())
        # painter.setPen(Qt.gray)
        # for r in self.refRects:
        # painter.drawRect(r)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'manuskript/ui/views/propertiesView_ui.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_propertiesView(object):
    def setupUi(self, propertiesView):
        propertiesView.setObjectName("propertiesView")
        propertiesView.resize(192, 186)
        self.verticalLayout = QtWidgets.QVBoxLayout(propertiesView)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.txtTitle = lineEditView(propertiesView)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.txtTitle.setFont(font)
        self.txtTitle.setStyleSheet("QLineEdit { background:transparent; }")
        self.txtTitle.setFrame(False)
        self.txtTitle.setObjectName("txtTitle")
        self.verticalLayout.addWidget(self.txtTitle)
        self.stack = QtWidgets.QStackedWidget(propertiesView)
        self.stack.setObjectName("stack")
        self.page = QtWidgets.QWidget()
        self.page.setObjectName("page")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.page)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.lblPOV = QtWidgets.QLabel(self.page)
        self.lblPOV.setObjectName("lblPOV")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.lblPOV)
        self.cmbPOV = cmbOutlineCharacterChoser(self.page)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmbPOV.sizePolicy().hasHeightForWidth())
        self.cmbPOV.setSizePolicy(sizePolicy)
        self.cmbPOV.setFrame(False)
        self.cmbPOV.setObjectName("cmbPOV")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.cmbPOV)
        self.lblStatus = QtWidgets.QLabel(self.page)
        self.lblStatus.setObjectName("lblStatus")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.lblStatus)
        self.cmbStatus = cmbOutlineStatusChoser(self.page)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmbStatus.sizePolicy().hasHeightForWidth())
        self.cmbStatus.setSizePolicy(sizePolicy)
        self.cmbStatus.setFrame(False)
        self.cmbStatus.setObjectName("cmbStatus")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.cmbStatus)
        self.lblLabel = QtWidgets.QLabel(self.page)
        self.lblLabel.setObjectName("lblLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.lblLabel)
        self.cmbLabel = cmbOutlineLabelChoser(self.page)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmbLabel.sizePolicy().hasHeightForWidth())
        self.cmbLabel.setSizePolicy(sizePolicy)
        self.cmbLabel.setFrame(False)
        self.cmbLabel.setObjectName("cmbLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.cmbLabel)
        self.lblCompile = QtWidgets.QLabel(self.page)
        self.lblCompile.setObjectName("lblCompile")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.lblCompile)
        self.chkCompile = chkOutlineCompile(self.page)
        self.chkCompile.setText("")
        self.chkCompile.setObjectName("chkCompile")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.chkCompile)
        self.lblGoal = QtWidgets.QLabel(self.page)
        self.lblGoal.setObjectName("lblGoal")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.lblGoal)
        self.txtGoal = lineEditView(self.page)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtGoal.sizePolicy().hasHeightForWidth())
        self.txtGoal.setSizePolicy(sizePolicy)
        self.txtGoal.setAutoFillBackground(False)
        self.txtGoal.setStyleSheet("border-radius: 6px;")
        self.txtGoal.setFrame(False)
        self.txtGoal.setObjectName("txtGoal")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.txtGoal)
        self.verticalLayout_2.addLayout(self.formLayout)
        self.stack.addWidget(self.page)
        self.page_2 = QtWidgets.QWidget()
        self.page_2.setObjectName("page_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.page_2)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setObjectName("formLayout_2")
        self.lblPOV_2 = QtWidgets.QLabel(self.page_2)
        self.lblPOV_2.setObjectName("lblPOV_2")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.lblPOV_2)
        self.cmbPOVMulti = cmbOutlineCharacterChoser(self.page_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmbPOVMulti.sizePolicy().hasHeightForWidth())
        self.cmbPOVMulti.setSizePolicy(sizePolicy)
        self.cmbPOVMulti.setFrame(False)
        self.cmbPOVMulti.setObjectName("cmbPOVMulti")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.cmbPOVMulti)
        self.label_31 = QtWidgets.QLabel(self.page_2)
        self.label_31.setObjectName("label_31")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_31)
        self.cmbStatusMulti = cmbOutlineStatusChoser(self.page_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmbStatusMulti.sizePolicy().hasHeightForWidth())
        self.cmbStatusMulti.setSizePolicy(sizePolicy)
        self.cmbStatusMulti.setFrame(False)
        self.cmbStatusMulti.setObjectName("cmbStatusMulti")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.cmbStatusMulti)
        self.label_34 = QtWidgets.QLabel(self.page_2)
        self.label_34.setObjectName("label_34")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_34)
        self.cmbLabelMulti = cmbOutlineLabelChoser(self.page_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmbLabelMulti.sizePolicy().hasHeightForWidth())
        self.cmbLabelMulti.setSizePolicy(sizePolicy)
        self.cmbLabelMulti.setFrame(False)
        self.cmbLabelMulti.setObjectName("cmbLabelMulti")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.cmbLabelMulti)
        self.label_35 = QtWidgets.QLabel(self.page_2)
        self.label_35.setObjectName("label_35")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_35)
        self.chkCompileMulti = chkOutlineCompile(self.page_2)
        self.chkCompileMulti.setText("")
        self.chkCompileMulti.setObjectName("chkCompileMulti")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.chkCompileMulti)
        self.label_36 = QtWidgets.QLabel(self.page_2)
        self.label_36.setObjectName("label_36")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.label_36)
        self.txtGoalMulti = lineEditView(self.page_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtGoalMulti.sizePolicy().hasHeightForWidth())
        self.txtGoalMulti.setSizePolicy(sizePolicy)
        self.txtGoalMulti.setAutoFillBackground(False)
        self.txtGoalMulti.setStyleSheet("border-radius: 6px;")
        self.txtGoalMulti.setFrame(False)
        self.txtGoalMulti.setObjectName("txtGoalMulti")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.txtGoalMulti)
        self.verticalLayout_3.addLayout(self.formLayout_2)
        self.stack.addWidget(self.page_2)
        self.verticalLayout.addWidget(self.stack)

        self.retranslateUi(propertiesView)
        self.stack.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(propertiesView)

    def retranslateUi(self, propertiesView):
        _translate = QtCore.QCoreApplication.translate
        propertiesView.setWindowTitle(_translate("propertiesView", "Form"))
        self.lblPOV.setText(_translate("propertiesView", "POV"))
        self.lblStatus.setText(_translate("propertiesView", "Status"))
        self.lblLabel.setText(_translate("propertiesView", "Label"))
        self.lblCompile.setText(_translate("propertiesView", "Compile"))
        self.lblGoal.setText(_translate("propertiesView", "Goal"))
        self.txtGoal.setPlaceholderText(_translate("propertiesView", "Word count"))
        self.lblPOV_2.setText(_translate("propertiesView", "POV"))
        self.label_31.setText(_translate("propertiesView", "Status"))
        self.label_34.setText(_translate("propertiesView", "Label"))
        self.label_35.setText(_translate("propertiesView", "Compile"))
        self.label_36.setText(_translate("propertiesView", "Goal"))
        self.txtGoalMulti.setPlaceholderText(_translate("propertiesView", "Word count"))

from manuskript.ui.views.chkOutlineCompile import chkOutlineCompile
from manuskript.ui.views.cmbOutlineCharacterChoser import cmbOutlineCharacterChoser
from manuskript.ui.views.cmbOutlineLabelChoser import cmbOutlineLabelChoser
from manuskript.ui.views.cmbOutlineStatusChoser import cmbOutlineStatusChoser
from manuskript.ui.views.lineEditView import lineEditView



#!/usr/bin/env python
# --!-- coding: utf8 --!--

"""Conf for models."""

import pytest

@pytest.fixture
def outlineModelBasic(MWEmptyProject):
    """Returns an outlineModel with a few items:
      * Folder
        * Text
      * Text
    """
    from manuskript.models import outlineItem
    mdl = MWEmptyProject.mdlOutline

    root = mdl.rootItem
    f = outlineItem(title="Folder", parent=root)
    t1 = outlineItem(title="Text", _type="md", parent=f)
    t2 = outlineItem(title="Text", _type="md", parent=root)

    return mdl



#!/usr/bin/env python
# --!-- coding: utf8 --!--

import os
from manuskript.importer.abstractImporter import abstractImporter
from manuskript.models import outlineItem
from manuskript.enums import Outline
from PyQt5.QtWidgets import qApp


class folderImporter(abstractImporter):

    name = "Folder"
    description = ""
    fileFormat = "<<folder>>"
    icon = "folder"

    @classmethod
    def isValid(cls):
        return True

    def startImport(self, filePath, parentItem, settingsWidget, fromString=None):
        """
        Imports from a folder.
        """
        ext = self.getSetting("ext").value()
        ext = [e.strip().replace("*", "").lower() for e in ext.split(",")]

        sorting = self.getSetting("sortItems").value()

        items = []
        stack = {}

        for dirpath, dirnames, filenames in os.walk(filePath):

            if dirpath in stack:
                item = stack[dirpath]
            else:
                # It's the parent folder, and we are not including it
                # so every item is attached to parentItem
                item = parentItem

            def addFile(f):
                fName, fExt = os.path.splitext(f)
                if fExt.lower() in ext:
                    try:
                        with open(os.path.join(dirpath, f), "r", encoding="utf-8") as fr:
                            content = fr.read()
                        child = outlineItem(title=fName, _type="md", parent=item)
                        child._data[Outline.text] = content
                        items.append(child)
                    except UnicodeDecodeError:
                        # Probably not a text file
                        pass

            def addFolder(d):
                child = outlineItem(title=d, parent=item)
                items.append(child)
                stack[os.path.join(dirpath, d)] = child

            if not self.getSetting("separateFolderFiles").value():
                # Import folder and files together (only makes differences if
                # they are sorted, really)
                allFiles = dirnames + filenames
                if sorting:
                    allFiles = sorted(allFiles)

                for f in allFiles:
                    if f in dirnames:
                        addFolder(f)
                    else:
                        addFile(f)

            else:
                # Import first folders, then files
                if sorting:
                    dirnames = sorted(dirnames)
                    filenames = sorted(filenames)

                # Import folders
                for d in dirnames:
                    addFolder(d)

                # Import files
                for f in filenames:
                    addFile(f)

        return items

    def settingsWidget(self, widget):
        """
        Takes a QWidget that can be modified and must be returned.
        """

        # Add group
        group = self.addGroup(widget.toolBox.widget(0),
                              qApp.translate("Import", "Folder import"))
        #group = cls.addPage(widget, "Folder import")

        self.addSetting("info", "label",
                        qApp.translate("Import", """<p><b>Info:</b> Imports a whole
                        directory structure. Folders are added as folders, and
                        plaintext documents within (you chose which ones by extension)
                        are added as scene.</p>
                        <p>Only text files are supported (not images, binary or others).</p>"""))

        self.addSetting("ext", "text",
                        qApp.translate("Import", "Include only those extensions:"),
                        default="*.txt, *.md",
                        tooltip=qApp.translate("Import", "Comma separated values")),

        self.addSetting("sortItems", "checkbox",
                        qApp.translate("Import", "Sort items by name"),
                        default=True),

        self.addSetting("separateFolderFiles", "checkbox",
                        qApp.translate("Import", "Import folder then files"),
                        default=True),

        self.addSettingsTo(group)

        return widget







# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'manuskript/ui/views/basicItemView_ui.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_basicItemView(object):
    def setupUi(self, basicItemView):
        basicItemView.setObjectName("basicItemView")
        basicItemView.resize(400, 425)
        self.verticalLayout = QtWidgets.QVBoxLayout(basicItemView)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_11.addItem(spacerItem)
        self.lblPlanPOV = QtWidgets.QLabel(basicItemView)
        self.lblPlanPOV.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lblPlanPOV.setObjectName("lblPlanPOV")
        self.horizontalLayout_11.addWidget(self.lblPlanPOV)
        self.cmbPOV = cmbOutlineCharacterChoser(basicItemView)
        self.cmbPOV.setFrame(False)
        self.cmbPOV.setObjectName("cmbPOV")
        self.horizontalLayout_11.addWidget(self.cmbPOV)
        self.lblGoal = QtWidgets.QLabel(basicItemView)
        self.lblGoal.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lblGoal.setObjectName("lblGoal")
        self.horizontalLayout_11.addWidget(self.lblGoal)
        self.txtGoal = lineEditView(basicItemView)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtGoal.sizePolicy().hasHeightForWidth())
        self.txtGoal.setSizePolicy(sizePolicy)
        self.txtGoal.setAutoFillBackground(False)
        self.txtGoal.setStyleSheet("border-radius: 6px;")
        self.txtGoal.setFrame(False)
        self.txtGoal.setObjectName("txtGoal")
        self.horizontalLayout_11.addWidget(self.txtGoal)
        self.verticalLayout.addLayout(self.horizontalLayout_11)
        self.txtSummarySentence = lineEditView(basicItemView)
        self.txtSummarySentence.setText("")
        self.txtSummarySentence.setObjectName("txtSummarySentence")
        self.verticalLayout.addWidget(self.txtSummarySentence)
        self.label_9 = QtWidgets.QLabel(basicItemView)
        self.label_9.setObjectName("label_9")
        self.verticalLayout.addWidget(self.label_9)
        self.txtSummaryFull = MDEditCompleter(basicItemView)
        self.txtSummaryFull.setObjectName("txtSummaryFull")
        self.verticalLayout.addWidget(self.txtSummaryFull)

        self.retranslateUi(basicItemView)
        QtCore.QMetaObject.connectSlotsByName(basicItemView)

    def retranslateUi(self, basicItemView):
        _translate = QtCore.QCoreApplication.translate
        basicItemView.setWindowTitle(_translate("basicItemView", "Form"))
        self.lblPlanPOV.setText(_translate("basicItemView", "POV:"))
        self.lblGoal.setText(_translate("basicItemView", "Goal:"))
        self.txtGoal.setPlaceholderText(_translate("basicItemView", "Word count"))
        self.txtSummarySentence.setPlaceholderText(_translate("basicItemView", "One line summary"))
        self.label_9.setText(_translate("basicItemView", "Few sentences summary:"))

from manuskript.ui.views.MDEditCompleter import MDEditCompleter
from manuskript.ui.views.cmbOutlineCharacterChoser import cmbOutlineCharacterChoser
from manuskript.ui.views.lineEditView import lineEditView



#!/usr/bin/env python
# --!-- coding: utf8 --!--

from manuskript.models.abstractModel import abstractModel


class outlineModel(abstractModel):
    def __init__(self, parent):
        abstractModel.__init__(self, parent)

    def findItemsByPOV(self, POV):
        "Returns a list of IDs of all items whose POV is ``POV``."
        return self.rootItem.findItemsByPOV(POV)



#!/usr/bin/env python
# --!-- coding: utf8 --!--

from manuskript.importer.abstractImporter import abstractImporter
from manuskript.exporter.pandoc import pandocExporter
from manuskript.importer.opmlImporter import opmlImporter
from manuskript.importer.markdownImporter import markdownImporter
from PyQt5.QtWidgets import qApp


class pandocImporter(abstractImporter):

    formatFrom = ""
    engine = "Pandoc"
    extraArgs = []

    @classmethod
    def isValid(cls):
        return pandocExporter().isValid()

    def startImport(self, filePath, parentItem, settingsWidget):

        formatTo = self.getSetting("formatTo").value().lower()
        wrap = self.getSetting("wrap").value().lower()

        # pandoc --from=markdown filename --to=opml --standalone
        args = [
            "--from={}".format(self.formatFrom),
            filePath,
            "--to={}".format(formatTo),
            "--wrap={}".format(wrap),
        ]

        if formatTo == "opml":
            args.append("--standalone")

        args += self.extraArgs

        r = pandocExporter().run(args)

        if r is None:
            return None

        if formatTo == "opml":
            return self.opmlImporter.startImport("", parentItem,
                                            settingsWidget, fromString=r)
        elif formatTo == "markdown":
            return self.mdImporter.startImport(filePath, parentItem,
                                                settingsWidget, fromString=r)

    def settingsWidget(self, widget):
        """
        Takes a QWidget that can be modified and must be returned.
        """

        # Add group
        group = self.addGroup(widget.toolBox.widget(0),
                              qApp.translate("Import", "Pandoc import"))

        self.addSetting("info", "label",
                        qApp.translate("Import", """<b>Info:</b> Manuskript can
                        import from <b>markdown</b> or <b>OPML</b>. Pandoc will
                        convert your document to either (see option below), and
                        then it will be imported in manuskript. One or the other
                        might give better result depending on your document.
                        <br/>&nbsp;"""))

        self.addSetting("formatTo", "combo",
                        qApp.translate("Import", "Import using:"),
                        vals="markdown|OPML")

        self.addSetting("wrap", "combo",
                        qApp.translate("Import", "Wrap lines:"),
                        vals="auto|none|preserve",
                        default="none",
                        tooltip=qApp.translate("Import", """<p>Should pandoc create
                        cosmetic / non-semantic line-breaks?</p><p>
                        <b>auto</b>: wraps at 72 characters.<br>
                        <b>none</b>: no line wrap.<br>
                        <b>preserve</b>: tries to preserves line wrap from the
                        original document.</p>"""))

        for s in self.settings:
            self.settings[s].widget(group)

        self.mdImporter = markdownImporter()
        widget = self.mdImporter.settingsWidget(widget)
        self.opmlImporter = opmlImporter()
        widget = self.opmlImporter.settingsWidget(widget)

        return widget


class markdownPandocImporter(pandocImporter):

    name = "Markdown"
    description = "Markdown, using pandoc"
    fileFormat = "Markdown files (*.md *.txt *)"
    icon = "text-x-markdown"
    formatFrom = "markdown"

class ePubPandocImporter(pandocImporter):

    name = "ePub"
    description = ""
    fileFormat = "ePub files (*.epub)"
    icon = "application-epub+zip"
    formatFrom = "epub"

class docXPandocImporter(pandocImporter):

    name = "DocX"
    description = ""
    fileFormat = "DocX files (*.docx)"
    icon = "application-vnd.openxmlformats-officedocument.wordprocessingml.document"
    formatFrom = "docx"

class odtPandocImporter(pandocImporter):

    name = "ODT"
    description = ""
    fileFormat = "Open Document files (*.odt)"
    icon = "application-vnd.oasis.opendocument.text"
    formatFrom = "odt"

class rstPandocImporter(pandocImporter):

    name = "reStructuredText"
    description = ""
    fileFormat = "reStructuredText files (*.rst)"
    icon = "text-plain"
    formatFrom = "rst"

class HTMLPandocImporter(pandocImporter):

    name = "HTML"
    description = ""
    fileFormat = "HTML files (*.htm *.html)"
    icon = "text-html"
    formatFrom = "html"

class LaTeXPandocImporter(pandocImporter):

    name = "LaTeX"
    description = ""
    fileFormat = "LaTeX files (*.tex)"
    icon = "text-x-tex"
    formatFrom = "latex"

class OPMLPandocImporter(pandocImporter):

    name = "OPML"
    description = ""
    fileFormat = "OPML files (*.opml *.xml)"
    icon = "text-x-opml+xml"
    formatFrom = "opml"



